/*
 * This file specifies whole configuration of sitegin.
 * It reads config file from content/config.toml it also parses command-line
 * arguments. CLI args take precedence over config options
 */
const cli = require('cli')
const fs = require('fs')
const path = require('path')
const toml = require('toml')

const options = cli.parse({
  noserver: ['n', 'Dont run server'],
  port: ['p', 'Port on which server should run', 'number', 1337],
  uiport: [null, 'BrowserSync UI port', 'number', 'port+1'],
  httpsonly: [null, 'Converts all http://ok1kvk.cz links to https'],
  httponly: [null, 'Converts all https://ok1kvk.cz links to http'],
  contentdir: [
    null,
    'Allows to specify arbitrary content directory.',
    'string',
    'content',
  ],
  themedir: [
    null,
    'Allows to specify arbitrary content directory.',
    'string',
    'theme',
  ],
  builddir: [
    null,
    'Allows to specify arbitrary content directory.',
    'string',
    null,
  ],
  staticdir: [
    null,
    'Allows to specify arbitrary directory for static files.',
    'string',
    'static',
  ],
  baseurl: [null, 'BaseURL without trailing /', 'string', ''],
  nowatch: [null, 'Disable watching for changes'],
  debugarticle: [
    null,
    'Article for which extra information should be printed out. Full file path ' +
      'relative to contentdir (Ex. articles/2010/hello-word.md)',
    'string',
    null,
  ],
})

module.exports = () => {
  console.log('Build step: Config')
  let builddir = 'build'
  if (options.builddir) ({ builddir } = options)
  else if (!options.noserver) builddir = 'build-debug'

  if (options.uiport === 'port+1') {
    options.uiport = options.port + 1
  }

  return new Promise(resolve => {
    const config = {
      options,
      builddir,
      sourceDir: options.contentdir,
      staticDir: options.staticdir,
      themeDir: options.themedir,
      articlesLocation: 'articles',
      redirectsLocation: 'redirects',
      linksPerPage: 6,
    }
    const configFile = path.join(config.sourceDir, 'config.toml')
    fs.readFile(configFile, 'utf-8', (err, data_) => {
      if (err) {
        console.log(`Failed to read configuration from ${configFile}`)
        console.log('Using defaults')
        resolve({ config })
        return
      }
      const data = toml.parse(data_)
      for (const attr of Object.keys(config)) {
        data[attr] = config[attr]
      }
      resolve({ config: data })
    })
  })
}
module.exports.watch = !options.noserver
module.exports.themeDir = options.themedir
module.exports.options = options
