const moment = require('moment')
const cli = require('cli')
const path = require('path')
const config = require('./config')

cli.main(function(args, options) {
  console.log(options)
  config()
    .then(function(obj) {
      sitegin(obj.config)
    })
    .catch(function(e) {
      console.log(e.stack)
      process.exit()
    })
})

function sitegin(config) {
  const options = config.options
  const pipeline = require('./sitegin')({
    watch: !options.noserver && !options.nowatch,
  })

  const sass = require('node-sass')
  const fs = require('fs')

  // Main builder function
  // If builder is not running - run it
  // If it is running - schedule rerun after it finishes
  let isRunning = false
  let runAgain = false
  let first = false
  let doSync = function() {}
  function run() {
    const startTime = moment()
    runAgain = false
    if (isRunning) {
      console.log(
        'Generator is still running. Queing another run after it finishes.',
      )
      runAgain = true
      return
    }
    console.log(
      '================================================================================',
    )
    console.log('Running generator')
    isRunning = true
    pipeline()
      .then(function() {
        isRunning = false
        console.log(
          'Generator finished in',
          moment().diff(startTime, 'seconds'),
          'seconds',
        )
        doSync()
        if (!first) {
          first = true
          copyStaticFiles(config.builddir, config.staticDir, config.themeDir)
        }
        if (runAgain) run()
      })
      .catch(function(e) {
        isRunning = false
        console.log(
          'Generator crashed in',
          moment().diff(startTime, 'seconds'),
          'seconds',
        )
        if (runAgain) run()
        if (e.stack) console.log(e.stack)
        else console.log(e)
        if (config.options.noserver) process.exit(13)
      })
  }

  console.log('Sitegin successfully loaded')
  run()

  rendersass(config.builddir, config.themeDir)

  if (!options.noserver) {
    const sync = require('browser-sync').create()
    doSync = function() {
      sync.reload('*')
    }
    sync.init({
      server: {
        baseDir: config.builddir,
      },
      ui: {
        port: options.port + 1,
      },
      port: options.port,
    })

    const chokidar = require('chokidar')
    // article or theme reload
    if (!options.nowatch) {
      const watches = [path.join(config.sourceDir, 'articles'), config.themeDir]
      console.log('Watching', watches, 'for changes')
      chokidar
        .watch(watches, { ignoreInitial: true })
        .on('all', function(event, path) {
          console.log('Content or theme changed. Rebuilding...')
          console.log('Change event:', event, 'on path:', path)
          run()
        })
    }
  }
}

function copyStaticFiles(builddir, staticdir, themedir) {
  // STATIC FILES
  const fsextra = require('node-fs-extra')
  fsextra.copy(
    staticdir,
    builddir,
    function(file) {
      return !(file.match('\\.git') || file.match('static/articles'))
    },
    function() {
      console.log('copy static done')
    },
  )
  fsextra.copy(`${staticdir}/articles`, `${builddir}/clanek`, function() {
    console.log('copy static/articles done')
  })
  fsextra.copy(
    `${themedir}/static`,
    builddir,
    function(file) {
      return !file.match('\\.git')
    },
    function() {
      console.log(`copy ${themedir}/static done`)
    },
  )
}

function rendersass(builddir, themedir) {
  // SASS
  const sass = require('node-sass')
  const fs = require('fs')
  const mkdirp = require('mkdirp')

  sass.render({ file: `${themedir}/sass/style.scss` }, function(err, result) {
    if (err === null) {
      console.log('compiled sass')
      mkdirp(`${builddir}/theme`, () => {
        fs.writeFile(`${builddir}/theme/style.css`, result.css, () => {})
      })
    } else console.log('error ', err)
  })
}
