/*
 * Transforms pages[0..length].content from markdown to HTML
 */

const marked = require('marked')

const renderer = new marked.Renderer()
const highlightjs = require('highlight.js')
const image = require('./image')
const toURL = require('./toURL')

let lang

const opts = {
  highlight(code) {
    let h
    if (lang) {
      if (highlightjs.getLanguage(lang))
        h = highlightjs.highlight(lang, code, true)
      else h = highlightjs.highlight(code)
    } else {
      h = highlightjs.highlightAuto(code)
    }
    return h.value
  },
  renderer,
  gfm: true,
  tables: true,
  breaks: false,
  pedantic: false,
  sanitize: false,
  smartLists: true,
  smartypants: false,
}

const parser = new marked.Parser(opts)
const lexer = new marked.Lexer(opts)

const renderImage = (href, title, text, curFilename, cfg) =>
  image(href, title, text, curFilename, cfg)

renderer.heading = (text, level, raw) =>
  `<h${level + 2} id="${toURL(raw)}">${text}</h${level + 2}>\n`

renderer.paragraph = text => {
  if (text.match('\\<img')) return `<div class="has-image">${text}</div>\n`
  return `<p>${text}</p>\n`
}

renderer._link = renderer.link

module.exports = obj => {
  console.log('Build step: Markdown')
  const { options } = obj.config
  renderer.link = (href_, title, text) => {
    let href = href_
    if (options.httponly) {
      if (href.match(/^https:\/\/ok1kvk.cz/))
        href = href.replace('https://', 'http://')
    }
    if (options.httpsonly) {
      if (href.match(/^http:\/\/ok1kvk.cz/))
        href = href.replace('http://', 'https://')
    }

    return renderer._link(href, title, text)
  }

  obj.pages.forEach(article => {
    renderer.image = (href, title, text) =>
      renderImage(href, title, text, article.file, obj.config)
    ;({ lang } = article.metadata)
    // eslint-disable-next-line no-param-reassign
    article.content = parser.parse(lexer.lex(article.content))
  })
  return Promise.resolve(obj)
}
