const toURL = require('./toURL')

module.exports = function tagsJob(obj) {
  console.log('Build step: Tags')
  return new Promise(resolve => {
    const tags = [
      {
        text: 'Všechny články a aktuality',
        url: 'tag/vse',
      },
    ]
    const tagPages = {
      'tag/vse': [],
    }
    for (const article of obj.pages) {
      if (!article.metadata.tags) {
        article.metadata.tags = []
      }
      if (article.metadata.tags) {
        const ntags = []
        for (const tag_ of article.metadata.tags) {
          const tag = {
            text: tag_,
            url: `tag/${toURL(tag_)}`,
          }
          ntags.push(tag)
          if (tags.map(e => e.url).indexOf(tag.url) < 0) tags.push(tag)
          if (tagPages[tag.url] === undefined) tagPages[tag.url] = []
          tagPages[tag.url].push(article)
        }

        if (!article.metadata.notarticle) {
          ntags.push(tags[0])
          tagPages[tags[0].url].push(article)
        }

        article.metadata.tags = ntags
      }
    }

    // eslint-disable-next-line no-param-reassign
    obj.tags = []
    // eslint-disable-next-line no-param-reassign
    if (!Array.isArray(obj.redirects)) obj.redirects = []

    const generateTagPages = (tag, pageList, subtags) => {
      const file = tag.url
      const pg = {}
      pg.type = 'redirect'
      pg.file = file
      pg.content = `${file}/1`
      obj.redirects.push(pg)

      pageList.sort((a, b) => {
        if (a.metadata.published === b.metadata.published) return 0
        if (a.metadata.published === undefined) return -1
        if (b.metadata.published === undefined) return 1
        const rhs = a.metadata.published
        const lhs = b.metadata.published

        return lhs > rhs ? 1 : lhs < rhs ? -1 : 0
      })
      let c = 0
      let p = 0
      let tagPage
      let prevurl
      let prevpage
      let firstpage
      let lastpage
      const tagPageList = []
      pageList.forEach(page => {
        if (c === 0) {
          p += 1
          if (tagPage !== undefined) {
            tagPage.metadata.nexturl = `${file}/${p}`
            obj.tags.push(tagPage)
            tagPageList.push(tagPage)
          }
          tagPage = { metadata: {} }
          if (firstpage === undefined) firstpage = tagPage
          lastpage = tagPage

          tagPage.file = `${file}/${p}/`
          tagPage.type = 'tag'
          tagPage.tag = tag
          tagPage.subtags = subtags
          tagPage.metadata.firstpage = firstpage
          tagPage.metadata.prevpage = prevpage
          tagPage.metadata.prevurl = prevurl
          tagPage.metadata.thispage = tagPage
          tagPage.metadata.pagenumber = p
          tagPage.canonicalURL = `https://ok1kvk.cz/${tagPage.file}`

          tagPage.content = []
          if (prevpage !== undefined) prevpage.metadata.nextpage = tagPage
          prevurl = tagPage.file
          prevpage = tagPage
        }
        tagPage.content.push(page)

        c += 1
        c %= obj.config.linksPerPage
      })
      obj.tags.push(tagPage)
      tagPageList.push(tagPage)
      for (const tp of tagPageList) {
        tp.metadata.lastpage = lastpage
      }
    }

    const gen = arr => arr.map(a => ({ text: a, url: `tag/${toURL(a)}` }))
    const subtags = {
      'tag/konstrukce': gen([
        'Konstrukce pro začátečníky',
        'HAM konstrukce',
        'Konstrukce s mikroprocesorem',
      ]),
      'tag/stalo-se': gen([
        'Akce pro veřejnost',
        'Napsali o nás',
        'Ocenění',
        'Soutěž',
        'SK',
        'Radioklub',
      ]),
      'tag/programovani': gen(['C/C++', 'Javascript', 'LaTeX', 'Nástroje']),
      'tag/mikroprocesory': gen(['AVR', 'Periferie', 'Knihovny']),
      'tag/technicke-clanky': gen(['Eagle']),
      'tag/zavody': gen(['HF', 'VHF']),
      'tag/aktuality': gen(['Pozvánky', 'Stalo se', 'Závody']),
      'tag/clanky': gen([
        'Programování',
        'Mikroprocesory',
        'Konstrukce',
        'Technické články',
        'Zajímavosti',
      ]),
      'tag/vse': gen(['Aktuality', 'Články']),
    }
    tags.forEach(tag => {
      generateTagPages(tag, tagPages[tag.url], subtags[tag.url])
    })
    resolve(obj)
  })
}
