const assert = require('assert')

const youtube = require('../utils/youtube.js')

describe('Youtube', () => {
  const links = {
    'https://www.youtube.com/watch?v=QH2-TGUlwu4': true,
    'http://www.youtube.com/watch?v=QH2-TGUlwu4': true,
    'https://youtu.be/QH2-TGUlwu4': true,
    'https://youtu.be/QH2-TGUlwu4?t=3m30s': true,
    'https://www.youtube.com/watch?t=3m30s&v=QH2-TGUlwu4': true,
    'https://www.youtube.cz/watch?t=3m30s&v=QH2-TGUlwu4': true,
    'https://www.youtube.com/watch?app=desktop&gl=CZ&v=QH2-TGUlwu4&hl=cs&v=cTa1Q2mBeJE': true,
    'https://www.youtube.com/': false,
    'https://www.youtube.com/channel/UCzNxmbk_LA1jXvYgivCCkpw': false,
    'youtube.png': false,
  }
  function testLink(link, val) {
    it(`isVideo ${link} => ${val}`, () => {
      assert(youtube.isVideo(link) === val)
    })
  }
  for (const [key, val] of Object.entries(links)) {
    testLink(key, val)
  }

  it('info https://youtu.be/wZZ7oFKsKzY?t=3h30m30s', () => {
    const info = youtube.info('https://youtu.be/wZZ7oFKsKzY?t=3h30m30s')
    assert(typeof info === 'object')
    assert(info.time === 3 * 3600 + 30 * 60 + 30)
    assert(info.videoid === 'wZZ7oFKsKzY')
  })

  it('info https://youtu.be/QH2-TGUlwu4', () => {
    const info = youtube.info('https://youtu.be/QH2-TGUlwu4')
    assert(typeof info === 'object')
    assert(info.time === 0)
    assert(info.videoid === 'QH2-TGUlwu4')
  })

  it('info https://www.youtube.com/watch?t=3m30s&v=QH2-TGUlwu4', () => {
    const info = youtube.info(
      'https://www.youtube.com/watch?t=3m30s&v=QH2-TGUlwu4',
    )
    assert(typeof info === 'object')
    assert(info.time === 3 * 60 + 30)
    assert(info.videoid === 'QH2-TGUlwu4')
  })
})
